// SPDX-License-Identifier: UNLICENSED
pragma solidity ^0.8.0;

import "./IERC20.sol";
import "./SafeMath.sol";
import "./Ownable.sol";
import "./ReentrancyGuard.sol";

contract BRDXCrowdsale is Ownable, ReentrancyGuard {
    using SafeMath for uint256;

    IERC20 public USDT;
    IERC20 public BRDX;
    uint256 public rate;
    uint256 public softCap;
    uint256 public hardCap;
    uint256 public startTime;
    uint256 public endTime;
    uint256 public cliffTime;
    bool public paused;
    mapping(address => uint256) public balances;
    address[] public buyers;

    event TokensPurchased(
        address indexed buyer,
        uint256 usdtAmount,
        uint256 tokenAmount
    );
    event TokensClaimed(address indexed buyer, uint256 tokenAmount);
    event FundsClaimed(address indexed owner, uint256 amount);
    event Withdrawal(address indexed buyer, uint256 usdtAmount);

    constructor(
        IERC20 _USDT,
        IERC20 _BRDX,
        uint256 _rate, // No need to input decimals, just the plain flat rate is needed
        uint256 _softCap,
        uint256 _hardCap,
        uint256 _startTime,
        uint256 _endTime
    ) {
        require(_rate > 0, "Rate must be greater than zero");
        require(_softCap > 0, "Soft cap must be greater than zero");
        require(_hardCap > _softCap, "Hard cap must be greater than soft cap");
        require(_startTime < _endTime, "Start time must be before end time");

        USDT = _USDT;
        BRDX = _BRDX;
        rate = _rate;
        softCap = _softCap;
        hardCap = _hardCap;
        startTime = _startTime;
        endTime = _endTime;
        paused = false;
    }

    modifier whenNotPaused() {
        require(!paused, "Contract is paused");
        _;
    }

    receive() external payable {
        revert("ETH not accepted");
    }

    function totalTokensSold() public view returns (uint256) {
        uint256 initialTokenBalance = BRDX.balanceOf(address(this));
        uint256 currentTokenBalance = initialTokenBalance;
        for (uint256 i = 0; i < buyers.length; i++) {
            currentTokenBalance = currentTokenBalance.add(balances[buyers[i]]);
        }
        return initialTokenBalance.sub(currentTokenBalance);
    }

    function viewBalances() external view returns (uint256, uint256) {
        uint256 usdtBalance = USDT.balanceOf(address(this));
        uint256 brdxBalance = BRDX.balanceOf(address(this));
        return (usdtBalance, brdxBalance);
    }

    function isSuccessful() public view returns (bool) {
        return totalTokensSold() >= softCap;
    }

    function setRate(uint256 _rate) external onlyOwner whenNotPaused {
        require(_rate > 0, "Rate must be greater than zero");
        rate = _rate;
    }

    function setCliffTime(uint256 _cliffTime) external onlyOwner whenNotPaused {
        require(
            _cliffTime >= startTime && _cliffTime <= endTime,
            "Cliff time must be within the crowdsale period"
        );
        cliffTime = _cliffTime;
    }

    function buyTokens(uint256 usdtAmount) external whenNotPaused nonReentrant {
        require(
            block.timestamp >= startTime && block.timestamp <= endTime,
            "Crowdsale not active"
        );
        require(usdtAmount > 0, "Invalid USDT amount");

        uint256 tokenAmount = usdtAmount.mul(rate);
        require(tokenAmount > 0, "Invalid BRDX amount");

        uint256 totalTokensSold = tokenAmount.add(balances[msg.sender]);
        require(totalTokensSold <= hardCap, "Exceeds hard cap");

        require(
            USDT.transferFrom(msg.sender, address(this), usdtAmount),
            "Transfer failed"
        );

        if (balances[msg.sender] == 0) {
            buyers.push(msg.sender);
        }

        balances[msg.sender] = totalTokensSold;

        emit TokensPurchased(msg.sender, usdtAmount, tokenAmount);
    }

    function claim() external whenNotPaused {
        require(block.timestamp >= cliffTime, "Cliff time not reached");
        require(balances[msg.sender] > 0, "No balance to claim");

        uint256 tokenAmount = balances[msg.sender];
        balances[msg.sender] = 0;
        require(
            BRDX.transfer(msg.sender, tokenAmount),
            "Token transfer failed"
        );

        emit TokensClaimed(msg.sender, tokenAmount);
    }

    function collectUSDT() external onlyOwner whenNotPaused {
        uint256 usdtBalance = USDT.balanceOf(address(this));
        require(
            usdtBalance > 0 || block.timestamp > endTime,
            "No funds to claim"
        );
        require(USDT.transfer(owner(), usdtBalance), "Transfer failed");
        emit FundsClaimed(owner(), usdtBalance);
    }

    function withdrawUnsoldTokens() external onlyOwner {
        require(block.timestamp > endTime, "Crowdsale not ended");

        uint256 unsoldTokens = BRDX.balanceOf(address(this)).sub(
            totalTokensSold()
        );
        require(unsoldTokens > 0, "No unsold tokens");

        require(BRDX.transfer(owner(), unsoldTokens), "Token transfer failed");
    }

    function endSale() external onlyOwner whenNotPaused {
        uint256 totalBuyers = buyers.length;

        for (uint256 i = 0; i < totalBuyers; i++) {
            address buyer = buyers[i];
            uint256 tokenAmount = balances[buyer];

            if (tokenAmount > 0) {
                balances[buyer] = 0;
                require(
                    BRDX.transfer(buyer, tokenAmount),
                    "Token transfer failed"
                );
                emit TokensClaimed(buyer, tokenAmount);
            }

            uint256 usdtAmount = USDT.balanceOf(address(this));
            if (usdtAmount > 0) {
                require(USDT.transfer(buyer, usdtAmount), "Transfer failed");
                emit Withdrawal(buyer, usdtAmount);
            }
        }

        endTime = block.timestamp; // Set the endTime to the current block timestamp
    }

    function pauseContract() external onlyOwner {
        paused = true;
    }

    function unpauseContract() external onlyOwner {
        paused = false;
    }
}
